<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Instagram_Model extends CI_Model {

	static private $endpoint = 'https://api.instagram.com/v1';
	private $access_token = '';

	public function __construct() {
		parent::__construct();
		$this->load->spark('curl/1.2.0');
		$this->db = $this->load->database('exmperiment_002', TRUE);
	}
	
	public function fetch_images_by_tag($tag = '') {
		# 500 pages ≈ 10000 photos
		$max_pages = 500; 
		
		$page = self::$endpoint.'/tags/'.$tag.'/media/recent?&access_token='.$this->access_token;
		
		$instagram = array();
		for ($i = 0; $i < $max_pages; $i++) {
			$result = @json_decode($this->curl->simple_get($page));
			if (!$result) {
				return FALSE;
			}
			$page = $result->pagination->next_url;
			foreach ($result->data as $obj) {
				$temp = new stdClass();
				$temp->id = $obj->id;
				$temp->url = $obj->images->thumbnail->url;
				array_push($instagram, $temp);
			}
		}

		$path = 'images/'.$tag.'/';
		
		if (!file_exists($path)) mkdir($path);

		foreach($instagram as $photo) {
			
			$tag_img_size = 30;
			
			$source_image = imagecreatefromstring($this->curl->simple_get($photo->url));
			
			if ($source_image) {
				$tag_img = imagecreatetruecolor($tag_img_size, $tag_img_size);
				imagecopyresampled($tag_img, $source_image, 0, 0, 0, 0, $tag_img_size, $tag_img_size, 
					imagesx($source_image), imagesy($source_image));

				$image_path = $path.$photo->id.'.jpg';
				imagejpeg($tag_img, $image_path, 85);

				$mini = imagecreatetruecolor(1, 1);
				imagecopyresampled($mini, $tag_img, 0, 0, 0, 0, 1, 1, $tag_img_size, $tag_img_size);
				$rgb = imagecolorat($mini, 0, 0);
				$average_color = imagecolorsforindex($mini, $rgb);

				imagedestroy($tag_img);
				imagedestroy($source_image);
				imagedestroy($mini);

				$image_data = array(
					'tag' => $tag,
					'instagram_id' => $photo->id,
					'path' => $image_path,
					'red' => $average_color['red'],
					'green' => $average_color['green'],
					'blue' => $average_color['blue']
				);

				$this->db->insert('small_images', $image_data);
			}
		}
	}
	
	public function fetch_latest_epmosaics() {		
		$page = self::$endpoint.'/tags/epmosaic/media/recent?access_token='.$this->access_token;
		$data = $this->curl->simple_get($page);
		$result = @json_decode($data);
		
		if ($result->data) {
			$instagram = array();
			foreach ($result->data as $img) {
				$temp = new stdClass();
				$temp->id = $img->id;
				$temp->instagram_link = $img->link;
				$temp->url = $img->images->standard_resolution->url;
				$temp->username = $img->user->username;
				array_push($instagram, $temp);
			}
			return $instagram[0];
		}
		return FALSE;
	}
	
}