<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Mosaic_Model extends CI_Model {
	
	public function __construct() {
		parent::__construct();
		$this->db = $this->load->database('exmperiment_002', TRUE);
		$this->load->helper('url');
	}
	
	public function get_latest_mosaic() {
		$this->db->order_by('timestamp', 'DESC');
		$this->db->limit(1);
		$query = $this->db->get('mosaic');
		if ($query->num_rows() > 0) {
			return $query->result();
		} 
		return FALSE;
	}
	
	public function get_mosaics($where = array()) {
		$this->db->where($where);
		$this->db->order_by('timestamp', 'DESC');
		$query = $this->db->get('mosaic');
		if($query->num_rows() > 0) {
			$result = $query->result();
			return $result;
		}
		return FALSE;
	}
	
	
	public function create_mosaic($instagram = '', $tag = '') {
		if (!$instagram || !$tag) return FALSE;

		$path = 'images/mosaic/';

		$tag_img_size = 30;
		$resolution_img_size = 40; 
		$gallery_img_size = 800;
		$mosaic_size = ($tag_img_size * $resolution_img_size);

		$path_original = $path.$instagram->id.'_original.jpg';
		$path_mosaic = $path.$instagram->id.'_mosaic.jpg';
		$path_mosaic_small = $path.$instagram->id.'_mosaic_small.jpg';

		# save original photo
		$instagram_img = imagecreatefromstring($this->curl->simple_get($instagram->url));
		imagejpeg($instagram_img, $path_original, 50);
		
		$source_img = imagecreatefromjpeg($path_original);
		
		$resolution_img = imagecreatetruecolor($resolution_img_size, $resolution_img_size);
		imagecopyresampled($resolution_img, $source_img, 0, 0, 0, 0, $resolution_img_size, $resolution_img_size, 
			imagesx($source_img), imagesy($source_img));
		
		imagedestroy($source_img);
		imagedestroy($instagram_img);

		$mosaic_img = imagecreatetruecolor($mosaic_size, $mosaic_size);
		
		# find matching tag images
		for ($y = 0; $y < $resolution_img_size; $y++){
			for ($x = 0; $x < $resolution_img_size; $x++) {
				$rgb = imagecolorat($resolution_img, $x, $y);
				$average_color = imagecolorsforindex($resolution_img, $rgb);	
				$small = $this->get_image_in_color_range($tag, $average_color);
				if ($small) {
					$tag_image = imagecreatefromjpeg(base_url().$small->path);
					imagecopymerge($mosaic_img, $tag_image, ($tag_img_size * $x), ($tag_img_size * $y), 0, 0, 
						$tag_img_size, $tag_img_size, 100);
					imagedestroy($tag_image);
				} else {
					$small = $this->get_image_near_color($tag, $average_color);
					if ($small) {
						$tag_image = imagecreatefromjpeg(base_url().$small->path);
						imagecopymerge($mosaic_img, $tag_image, ($tag_img_size * $x), ($tag_img_size * $y), 0, 0, 
							$tag_img_size, $tag_img_size, 100);
						imagedestroy($tag_image);
					}
				}
			}
		}

		imagejpeg($mosaic_img, $path_mosaic, 50);

		$gallery_img = imagecreatetruecolor($gallery_img_size, $gallery_img_size);
		imagecopyresampled($gallery_img, $mosaic_img, 0, 0, 0, 0, $gallery_img_size, $gallery_img_size, 
			$mosaic_size, $mosaic_size);
		imagejpeg($gallery_img, $path_mosaic_small, 50);
		imagedestroy($gallery_img);
		imagedestroy($mosaic_img);
		
		$data = array(
			'username' =>  $instagram->username,
			'instagram_img_id' => $instagram->id,
			'instagram_link' => $instagram->instagram_link,
			'mosaic' => $instagram->id.'_mosaic.jpg',
			'mosaic_small' => $instagram->id.'_mosaic_small.jpg',
			'original' => $instagram->id.'_original.jpg',
			'tag' => $tag
		);
		
		$this->db->insert('mosaic', $data);
		return $path_mosaic;
	}
	
	public function get_image_in_color_range($tag = '', $color = array()) {
		$margin = 40;
		$sql ='SELECT * FROM small_images WHERE tag=\''.$tag.'\' AND (red BETWEEN '.($color['red']-$margin).
			' AND '.($color['red']+$margin).') AND (green BETWEEN '.($color['green']-$margin).' AND '.
			($color['green']+$margin).') AND (blue BETWEEN '.($color['blue']-$margin).' AND '.($color['blue']+$margin).')  
			ORDER BY rand() LIMIT 1';
		$query = $this->db->query($sql);
		if ($query->num_rows() > 0) {
			$result = $query->result();
			return $result[0];
		} 
		return FALSE;
	}
	
	public function get_image_near_color($tag = '', $color = array()) {
		$this->db->select('path, red, green, blue, ABS('.$color['red'].'-red) AS r, 
			ABS('.$color['green'].'-green) AS g, ABS('.$color['blue'].'-blue) AS b');
		$this->db->where('tag', $tag);
		$this->db->order_by('r, g, b');
		$this->db->limit(1);
		$query = $this->db->get('small_images');
		if ($query->num_rows() > 0) {
			$result = $query->result();
			return $result[0];
		} 
		return FALSE;
	}
	
}
